@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 Derick Rethans                               |
+   | Copyright (c) 1997-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -65,7 +65,12 @@ void vld_only_leave_first_catch(zend_op_array *opa, vld_branch_info *branch_info
 	if (opa->opcodes[position].opcode == ZEND_FETCH_CLASS) {
 		position++;
 	}
+
+#if PHP_VERSION_ID >= 70100
+	exit_jmp = position + ((signed int) opa->opcodes[position].extended_value / sizeof(zend_op));
+#else
 	exit_jmp = opa->opcodes[position].extended_value;
+#endif
 
 	if (opa->opcodes[position].opcode != ZEND_CATCH) {
 		return;
@@ -90,7 +95,11 @@ void vld_branch_post_process(zend_op_array *opa, vld_branch_info *branch_info)
 	 * considered entry points */
 	for (i = 0; i < branch_info->entry_points->size; i++) {
 		if (vld_set_in(branch_info->entry_points, i) && opa->opcodes[i].opcode == ZEND_CATCH) {
+#if PHP_VERSION_ID >= 70100
+			vld_only_leave_first_catch(opa, branch_info, i + ((signed int) opa->opcodes[i].extended_value / sizeof(zend_op)));
+#else
 			vld_only_leave_first_catch(opa, branch_info, opa->opcodes[i].extended_value);
+#endif
 		}
 	}
 
@@ -176,7 +185,8 @@ static int vld_path_exists(vld_path *path, unsigned int elem1, unsigned int elem
 
 static void vld_branch_find_path(unsigned int nr, vld_branch_info *branch_info, vld_path *prev_path)
 {
-	unsigned int out0, out1, last;
+	int out0, out1;
+	unsigned int last;
 	vld_path *new_path;
 	int found = 0;
 
@@ -220,10 +230,10 @@ void vld_branch_find_paths(vld_branch_info *branch_info)
 void vld_branch_info_dump(zend_op_array *opa, vld_branch_info *branch_info TSRMLS_DC)
 {
 	unsigned int i, j;
-	char *fname = opa->function_name ? ZSTRING_VALUE(opa->function_name) : "__main";
+	const char *fname = opa->function_name ? ZSTRING_VALUE(opa->function_name) : "__main";
 
 	if (VLD_G(path_dump_file)) {
-		fprintf(VLD_G(path_dump_file), "subgraph cluster_%08x {\n\tlabel=\"%s\";\n\tgraph [rankdir=\"LR\"];\n\tnode [shape = record];\n", opa, fname, fname);
+		fprintf(VLD_G(path_dump_file), "subgraph cluster_%p {\n\tlabel=\"%s\";\n\tgraph [rankdir=\"LR\"];\n\tnode [shape = record];\n", opa, fname);
 
 		for (i = 0; i < branch_info->starts->size; i++) {
 			if (vld_set_in(branch_info->starts, i)) {
