@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 Derick Rethans                               |
+   | Copyright (c) 1997-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -24,7 +24,7 @@ typedef struct _vld_branch {
 	unsigned int start_lineno;
 	unsigned int end_lineno;
 	unsigned int end_op;
-	unsigned int out[2];
+	int          out[2];
 } vld_branch;
 
 typedef struct _vld_path {
