@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 Derick Rethans                               |
+   | Copyright (c) 1997-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -15,6 +15,7 @@
 
 #include "php.h"
 #include "zend_alloc.h"
+#include "zend_compile.h"
 #include "branchinfo.h"
 #include "srm_oparray.h"
 #include "ext/standard/url.h"
@@ -23,6 +24,12 @@
 
 ZEND_EXTERN_MODULE_GLOBALS(vld)
 
+#if ZEND_USE_ABS_JMP_ADDR
+# define VLD_ZNODE_JMP_LINE(node, opline, base)  (int32_t)(((long)((node).jmp_addr) - (long)(base_address)) / sizeof(zend_op))
+#else
+# define VLD_ZNODE_JMP_LINE(node, opline, base)  (int32_t)(((int32_t)((node).jmp_offset) / sizeof(zend_op)) + (opline))
+#endif
+
 /* Input zend_compile.h
  * And replace [^(...)(#define )([^ \t]+).*$]
  * BY     [/=*  \1 *=/  { "\3", ALL_USED },] REMEMBER to remove the two '=' signs
@@ -49,7 +56,7 @@ static const op_usage opcodes[] = {
 	/*  18 */	{ "IS_NOT_EQUAL", ALL_USED },
 	/*  19 */	{ "IS_SMALLER", ALL_USED },
 	/*  20 */	{ "IS_SMALLER_OR_EQUAL", ALL_USED },
-	/*  21 */	{ "CAST", ALL_USED },
+	/*  21 */	{ "CAST", ALL_USED | EXT_VAL },
 	/*  22 */	{ "QM_ASSIGN", RES_USED | OP1_USED },
 	/*  23 */	{ "ASSIGN_ADD", ALL_USED | EXT_VAL },
 	/*  24 */	{ "ASSIGN_SUB", ALL_USED | EXT_VAL },
@@ -69,7 +76,13 @@ static const op_usage opcodes[] = {
 	/*  38 */	{ "ASSIGN", ALL_USED },
 	/*  39 */	{ "ASSIGN_REF", SPECIAL },
 	/*  40 */	{ "ECHO", OP1_USED },
+#if PHP_VERSION_ID < 70000
 	/*  41 */	{ "PRINT", RES_USED | OP1_USED },
+#elif PHP_VERSION_ID < 70100
+	/*  41 */	{ "UNKNOWN", ALL_USED },
+#else
+	/*  41 */	{ "GENERATOR_CREATE", RES_USED },
+#endif
 	/*  42 */	{ "JMP", OP1_USED | OP1_OPLINE },
 	/*  43 */	{ "JMPZ", OP1_USED | OP2_USED | OP2_OPLINE },
 	/*  44 */	{ "JMPNZ", OP1_USED | OP2_USED | OP2_OPLINE },
@@ -81,20 +94,35 @@ static const op_usage opcodes[] = {
 	/*  50 */	{ "BRK", SPECIAL },
 	/*  51 */	{ "CONT", ALL_USED },
 	/*  52 */	{ "BOOL", RES_USED | OP1_USED },
+#if defined(ZEND_ENGINE_3)
+	/*  53 */	{ "FAST_CONCAT", ALL_USED },
+	/*  54 */	{ "ROPE_INIT", ALL_USED | EXT_VAL },
+	/*  55 */	{ "ROPE_ADD", ALL_USED | EXT_VAL },
+	/*  56 */	{ "ROPE_END", ALL_USED | EXT_VAL },
+#else
 	/*  53 */	{ "INIT_STRING", RES_USED },
 	/*  54 */	{ "ADD_CHAR", ALL_USED },
 	/*  55 */	{ "ADD_STRING", ALL_USED },
 	/*  56 */	{ "ADD_VAR", ALL_USED },
+#endif
 	/*  57 */	{ "BEGIN_SILENCE", ALL_USED },
 	/*  58 */	{ "END_SILENCE", ALL_USED },
 	/*  59 */	{ "INIT_FCALL_BY_NAME", SPECIAL },
 	/*  60 */	{ "DO_FCALL", SPECIAL },
+#if defined(ZEND_ENGINE_3)
+	/*  61 */	{ "INIT_FCALL", ALL_USED },
+#else
 	/*  61 */	{ "DO_FCALL_BY_NAME", SPECIAL },
+#endif
 	/*  62 */	{ "RETURN", OP1_USED },
 	/*  63 */	{ "RECV", RES_USED | OP1_USED },
 	/*  64 */	{ "RECV_INIT", ALL_USED },
 	/*  65 */	{ "SEND_VAL", OP1_USED },
+#if defined(ZEND_ENGINE_3)
+	/*  66 */	{ "SEND_VAR_EX", ALL_USED },
+#else
 	/*  66 */	{ "SEND_VAR", OP1_USED },
+#endif
 	/*  67 */	{ "SEND_REF", ALL_USED },
 	/*  68 */	{ "NEW", SPECIAL },
 #if (PHP_MAJOR_VERSION < 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 1)
@@ -111,15 +139,20 @@ static const op_usage opcodes[] = {
 	/*  72 */	{ "ADD_ARRAY_ELEMENT", ALL_USED },
 	/*  73 */	{ "INCLUDE_OR_EVAL", ALL_USED | OP2_INCLUDE },
 	/*  74 */	{ "UNSET_VAR", ALL_USED },
-#ifdef ZEND_ENGINE_2
+#if defined(ZEND_ENGINE_2) || defined(ZEND_ENGINE_3)
 	/*  75 */	{ "UNSET_DIM", ALL_USED },
 	/*  76 */	{ "UNSET_OBJ", ALL_USED },
 #else
 	/*  75 */	{ "UNSET_DIM_OBJ", ALL_USED },
 	/*  76 */	{ "ISSET_ISEMPTY", ALL_USED },
 #endif
+#if defined(ZEND_ENGINE_3)
+	/*  77 */	{ "FE_RESET_R", SPECIAL },
+	/*  78 */	{ "FE_FETCH_R", ALL_USED | EXT_VAL_JMP_REL },
+#else
 	/*  77 */	{ "FE_RESET", SPECIAL },
 	/*  78 */	{ "FE_FETCH", ALL_USED | OP2_OPNUM },
+#endif
 	/*  79 */	{ "EXIT", ALL_USED },
 	/*  80 */	{ "FETCH_R", RES_USED | OP1_USED | OP_FETCH },
 	/*  81 */	{ "FETCH_DIM_R", ALL_USED },
@@ -139,7 +172,11 @@ static const op_usage opcodes[] = {
 	/*  95 */	{ "FETCH_UNSET", ALL_USED },
 	/*  96 */	{ "FETCH_DIM_UNSET", ALL_USED },
 	/*  97 */	{ "FETCH_OBJ_UNSET", ALL_USED },
+#if defined(ZEND_ENGINE_3)
+	/*  98 */	{ "FETCH_LIST", ALL_USED },
+#else
 	/*  98 */	{ "FETCH_DIM_TMP_VAR", ALL_USED },
+#endif
 	/*  99 */	{ "FETCH_CONSTANT", ALL_USED },
 #if (PHP_MAJOR_VERSION < 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 3)
 	/*  100 */	{ "DECLARE_FUNCTION_OR_CLASS", ALL_USED },
@@ -152,14 +189,11 @@ static const op_usage opcodes[] = {
 	/*  104 */	{ "EXT_NOP", ALL_USED },
 	/*  105 */	{ "TICKS", ALL_USED },
 	/*  106 */	{ "SEND_VAR_NO_REF", ALL_USED | EXT_VAL },
-#ifdef ZEND_ENGINE_2
-	/*  107 */	{ "CATCH", ALL_USED | EXT_VAL },
+#if PHP_VERSION_ID >= 50000
+	/*  107 */	{ "CATCH", SPECIAL },
 	/*  108 */	{ "THROW", ALL_USED | EXT_VAL },
-	
 	/*  109 */	{ "FETCH_CLASS", SPECIAL },
-	
 	/*  110 */	{ "CLONE", ALL_USED },
-
 #if (PHP_MAJOR_VERSION < 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION <= 2)
 	/*  111 */	{ "INIT_CTOR_CALL", ALL_USED },
 #else 
@@ -171,17 +205,35 @@ static const op_usage opcodes[] = {
 #endif
 	/*  112 */	{ "INIT_METHOD_CALL", ALL_USED },
 	/*  113 */	{ "INIT_STATIC_METHOD_CALL", ALL_USED },
-	
 	/*  114 */	{ "ISSET_ISEMPTY_VAR", ALL_USED | EXT_VAL },
 	/*  115 */	{ "ISSET_ISEMPTY_DIM_OBJ", ALL_USED | EXT_VAL },
 	
+#if defined(ZEND_ENGINE_3)
+	/*  116 */	{ "SEND_VAL_EX", ALL_USED },
+	/*  117 */	{ "SEND_VAR", ALL_USED },
+	/*  118 */	{ "INIT_USER_CALL", ALL_USED | EXT_VAL },
+#else
 	/*  116 */	{ "IMPORT_FUNCTION", ALL_USED },
 	/*  117 */	{ "IMPORT_CLASS", ALL_USED },
 	/*  118 */	{ "IMPORT_CONST", ALL_USED },
+#endif
 	
 	/*  119 */	{ "UNKNOWN", ALL_USED },
 	/*  120 */	{ "UNKNOWN", ALL_USED },
 	
+#if defined(ZEND_ENGINE_3)
+	/*  121 */	{ "STRLEN", ALL_USED },
+	/*  122 */	{ "DEFINED", ALL_USED },
+	/*  123 */	{ "TYPE_CHECK", ALL_USED | EXT_VAL },
+	/*  124 */	{ "VERIFY_RETURN_TYPE", ALL_USED },
+	/*  125 */	{ "FE_RESET_RW", SPECIAL },
+	/*  126 */	{ "FE_FETCH_RW", ALL_USED | EXT_VAL_JMP_REL },
+	/*  127 */	{ "FE_FREE", ALL_USED },
+	/*  128 */	{ "INIT_DYNAMIC_CALL", ALL_USED },
+	/*  129 */	{ "DO_ICALL", ALL_USED },
+	/*  130 */	{ "DO_UCALL", ALL_USED },
+	/*  131 */	{ "DO_FCALL_BY_NAME", SPECIAL },
+#else
 	/*  121 */	{ "ASSIGN_ADD_OBJ", ALL_USED },
 	/*  122 */	{ "ASSIGN_SUB_OBJ", ALL_USED },
 	/*  123 */	{ "ASSIGN_MUL_OBJ", ALL_USED },
@@ -193,25 +245,20 @@ static const op_usage opcodes[] = {
 	/*  129 */	{ "ASSIGN_BW_OR_OBJ", ALL_USED },
 	/*  130 */	{ "ASSIGN_BW_AND_OBJ", ALL_USED },
 	/*  131 */	{ "ASSIGN_BW_XOR_OBJ", ALL_USED },
+#endif
 
 	/*  132 */	{ "PRE_INC_OBJ", ALL_USED },
 	/*  133 */	{ "PRE_DEC_OBJ", ALL_USED },
 	/*  134 */	{ "POST_INC_OBJ", ALL_USED },
 	/*  135 */	{ "POST_DEC_OBJ", ALL_USED },
-
 	/*  136 */	{ "ASSIGN_OBJ", ALL_USED },
 	/*  137 */	{ "OP_DATA", ALL_USED },
-	
 	/*  138 */	{ "INSTANCEOF", ALL_USED },
-	
 	/*  139 */	{ "DECLARE_CLASS", ALL_USED },
 	/*  140 */	{ "DECLARE_INHERITED_CLASS", ALL_USED },
 	/*  141 */	{ "DECLARE_FUNCTION", ALL_USED },
-	
 	/*  142 */	{ "RAISE_ABSTRACT_ERROR", ALL_USED },
-	
 	/*  143 */	{ "DECLARE_CONST", OP1_USED | OP2_USED },
-	
 	/*  144 */	{ "ADD_INTERFACE", ALL_USED },
 	/*  145 */	{ "VERIFY_INSTANCEOF", ALL_USED },
 	/*  146 */	{ "VERIFY_ABSTRACT_CLASS", ALL_USED },
@@ -219,26 +266,58 @@ static const op_usage opcodes[] = {
 	/*  148 */	{ "ISSET_ISEMPTY_PROP_OBJ", ALL_USED },
 	/*  149 */	{ "HANDLE_EXCEPTION", NONE_USED },
 	/*  150 */	{ "USER_OPCODE", ALL_USED },
+#if defined(ZEND_ENGINE_3)
+	/*  151 */	{ "ASSERT_CHECK", ALL_USED },
+#else
 	/*  151 */	{ "UNKNOWN", ALL_USED },
+#endif
 	/*  152 */	{ "JMP_SET", ALL_USED | OP2_OPLINE },
 	/*  153 */	{ "DECLARE_LAMBDA_FUNCTION", OP1_USED },
 	/*  154 */	{ "ADD_TRAIT", ALL_USED },
 	/*  155 */	{ "BIND_TRAITS", OP1_USED },
 	/*  156 */	{ "SEPARATE", OP1_USED | RES_USED },
+#if defined(ZEND_ENGINE_3)
+	/*  157 */	{ "FETCH_CLASS_NAME", ALL_USED },
+#else
 	/*  157 */	{ "QM_ASSIGN_VAR", OP1_USED | RES_USED },
+#endif
 	/*  158 */	{ "JMP_SET_VAR", OP1_USED | RES_USED },
 	/*  159 */	{ "DISCARD_EXCEPTION", NONE_USED },
 	/*  160 */	{ "YIELD", ALL_USED },
 	/*  161 */	{ "GENERATOR_RETURN", NONE_USED },
 	/*  162 */	{ "FAST_CALL", SPECIAL },
 	/*  163 */	{ "FAST_RET", SPECIAL },
-	/*  164 */	{ "ZEND_RECV_VARIADIC", ALL_USED },
-	/*  165 */	{ "POW", ALL_USED },
-	/*  166 */	{ "ASSIGN_POW", ALL_USED | EXT_VAL },
+	/*  164 */	{ "RECV_VARIADIC", ALL_USED },
+	/*  165 */	{ "SEND_UNPACK", ALL_USED },
+	/*  166 */	{ "POW", ALL_USED },
+	/*  167 */	{ "ASSIGN_POW", ALL_USED },
+	/*  168 */	{ "BIND_GLOBAL", ALL_USED },
+	/*  169 */	{ "COALESCE", ALL_USED },
+	/*  170 */	{ "SPACESHIP", ALL_USED },
+#endif
+#if PHP_VERSION_ID >= 70100
+    /*  171 */ { "DECLARE_ANON_CLASS", OP2_USED | RES_USED | RES_CLASS },
+    /*  172 */ { "DECLARE_ANON_INHERITED_CLASS", OP2_USED | RES_USED | RES_CLASS },
+    /*  173 */ { "FETCH_STATIC_PROP_R", RES_USED | OP1_USED | OP_FETCH },
+    /*  174 */ { "FETCH_STATIC_PROP_W", RES_USED | OP1_USED | OP_FETCH },
+    /*  175 */ { "FETCH_STATIC_PROP_RW", RES_USED | OP1_USED | OP_FETCH },
+    /*  176 */ { "FETCH_STATIC_PROP_IS", ALL_USED },
+    /*  177 */ { "FETCH_STATIC_PROP_FUNC_ARG", RES_USED | OP1_USED | OP_FETCH },
+    /*  178 */ { "FETCH_STATIC_PROP_UNSET", ALL_USED },
+    /*  179 */ { "UNSET_STATIC_PROP", ALL_USED },
+    /*  180 */ { "ISSET_ISEMPTY_STATIC_PROP", RES_USED | OP1_USED },
+    /*  181 */ { "FETCH_CLASS_CONSTANT", ALL_USED },
+    /*  182 */ { "BIND_LEXICAL", ALL_USED },
+    /*  183 */ { "BIND_STATIC", ALL_USED },
+	/*  184 */	{ "FETCH_THIS", ALL_USED },
+	/*  185 */	{ "UNKNOWN", ALL_USED },
+	/*  186 */	{ "ISSET_ISEMPTY_THIS", ALL_USED },
 #endif
 };
 
+#if PHP_VERSION_ID < 70000
 zend_brk_cont_element* vld_find_brk_cont(int nest_levels, int array_offset, zend_op_array *op_array);
+#endif
 
 static inline int vld_dump_zval_null(ZVAL_VALUE_TYPE value)
 {
@@ -258,7 +337,10 @@ static inline int vld_dump_zval_double(ZVAL_VALUE_TYPE value)
 static inline int vld_dump_zval_string(ZVAL_VALUE_TYPE value)
 {
 	ZVAL_VALUE_STRING_TYPE *new_str;
-	int new_len, len;
+#if PHP_VERSION_ID < 70000
+	int new_len;
+#endif
+	int len;
 
 	new_str = php_url_encode(ZVAL_STRING_VALUE(value), ZVAL_STRING_LEN(value) PHP_URLENCODE_NEW_LEN(new_len));
 	len = vld_printf (stderr, "'%s'", ZSTRING_VALUE(new_str));
@@ -313,6 +395,30 @@ static inline int vld_dump_zval_constant_array(ZVAL_VALUE_TYPE value)
 }
 #endif
 
+static inline int vld_dump_zval_undef(ZVAL_VALUE_TYPE value)
+{
+	return vld_printf (stderr, "<undef>");
+}
+
+static inline int vld_dump_zval_reference(ZVAL_VALUE_TYPE value)
+{
+	return vld_printf (stderr, "<reference>");
+}
+
+static inline int vld_dump_zval_callable(ZVAL_VALUE_TYPE value)
+{
+	return vld_printf (stderr, "<callable>");
+}
+
+static inline int vld_dump_zval_indirect(ZVAL_VALUE_TYPE value)
+{
+	return vld_printf (stderr, "<indirect>");
+}
+
+static inline int vld_dump_zval_ptr(ZVAL_VALUE_TYPE value)
+{
+	return vld_printf (stderr, "<ptr>");
+}
 
 int vld_dump_zval (zval val)
 {
@@ -350,7 +456,7 @@ int vld_dump_zval (zval val)
 }
 
 
-int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsigned int base_address TSRMLS_DC)
+int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsigned int base_address, zend_op_array *op_array, int opline TSRMLS_DC)
 {
 	int len = 0;
 
@@ -371,7 +477,7 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsi
 			VLD_PRINT1(3, " IS_CONST (%d) ", VLD_ZNODE_ELEM(node, var) / sizeof(temp_variable));
 #endif
 #if PHP_VERSION_ID >= 70000
-			vld_dump_zval(*node.zv);
+			vld_dump_zval(*RT_CONSTANT_EX(op_array->literals, node));
 #else
 # if PHP_VERSION_ID >= 50399
 			vld_dump_zval(*node.zv);
@@ -380,7 +486,7 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsi
 # endif
 #endif
 			break;
-#ifdef ZEND_ENGINE_2
+#if defined(ZEND_ENGINE_2) || defined(ZEND_ENGINE_3)
 		case IS_TMP_VAR: /* 2 */
 			VLD_PRINT(3, " IS_TMP_VAR ");
 			len += vld_printf (stderr, "~%d", VAR_NUM(VLD_ZNODE_ELEM(node, var)));
@@ -401,14 +507,14 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsi
 #endif
 		case VLD_IS_OPNUM:
 #if PHP_VERSION_ID >= 70000
-			len += vld_printf (stderr, "->%d", ((long) VLD_ZNODE_ELEM(node, jmp_addr) - base_address) / sizeof(zend_op));
+			len += vld_printf (stderr, "->%d", VLD_ZNODE_JMP_LINE(node, opline, base_address));
 #else
 			len += vld_printf (stderr, "->%d", VLD_ZNODE_ELEM(node, opline_num));
 #endif
 			break;
 		case VLD_IS_OPLINE:
 #if PHP_VERSION_ID >= 70000
-			len += vld_printf (stderr, "->%d", ((long) VLD_ZNODE_ELEM(node, jmp_addr) - base_address) / sizeof(zend_op));
+			len += vld_printf (stderr, "->%d", VLD_ZNODE_JMP_LINE(node, opline, base_address));
 #else
 			len += vld_printf (stderr, "->%d", (VLD_ZNODE_ELEM(node, opline_num) - base_address) / sizeof(zend_op));
 #endif
@@ -416,7 +522,7 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsi
 		case VLD_IS_CLASS:
 			len += vld_printf (stderr, ":%d", VAR_NUM(VLD_ZNODE_ELEM(node, var)));
 			break;
-#else
+#else /* ZEND_ENGINE_1 */
 		case IS_TMP_VAR: /* 2 */
 			len += vld_printf (stderr, "~%d", node.u.var);
 			break;
@@ -442,7 +548,12 @@ static unsigned int vld_get_special_flags(const zend_op *op, unsigned int base_a
 	unsigned int flags = 0;
 
 	switch (op->opcode) {
+#if PHP_VERSION_ID >= 70000
+		case ZEND_FE_RESET_R:
+		case ZEND_FE_RESET_RW:
+#else
 		case ZEND_FE_RESET:
+#endif
 			flags = ALL_USED;
 #if (PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1)
 			flags |= OP2_OPNUM;
@@ -489,7 +600,7 @@ static unsigned int vld_get_special_flags(const zend_op *op, unsigned int base_a
 			break;
 #endif
 
-#ifdef ZEND_ENGINE_2
+#if defined(ZEND_ENGINE_2) || defined(ZEND_ENGINE_3)
 		case ZEND_FETCH_CLASS:
 			flags = EXT_VAL|RES_USED|OP2_USED|RES_CLASS;
 			break;
@@ -497,7 +608,7 @@ static unsigned int vld_get_special_flags(const zend_op *op, unsigned int base_a
 
 		case ZEND_NEW:
 			flags = RES_USED|OP1_USED;
-#ifdef ZEND_ENGINE_2
+#if defined(ZEND_ENGINE_2) || defined(ZEND_ENGINE_3)
 			flags |= OP1_CLASS;
 #endif
 			break;
@@ -518,6 +629,18 @@ static unsigned int vld_get_special_flags(const zend_op *op, unsigned int base_a
 				flags = OP2_USED|OP2_OPNUM|EXT_VAL;
 			}
 			break;
+#endif
+#if PHP_VERSION_ID >= 50000
+		case ZEND_CATCH:
+			flags = ALL_USED;
+			if (!op->result.num) {
+#if PHP_VERSION_ID >= 70100
+				flags |= EXT_VAL_JMP_REL;
+#else
+				flags |= EXT_VAL_JMP_ABS;
+#endif
+			}
+			break;
 #endif
 	}
 	return flags;
@@ -569,8 +692,25 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 		res_type = VLD_IS_CLASS;
 	}
 
+#if PHP_VERSION_ID >= 50600 && PHP_VERSION_ID < 70100
+	switch (op.opcode) {
+		case ZEND_FAST_RET:
+			if (op.extended_value == ZEND_FAST_RET_TO_FINALLY) {
+				fetch_type = "to_finally";
+			} else if (op.extended_value == ZEND_FAST_RET_TO_CATCH) {
+				fetch_type = "to_catch";
+			}
+			break;
+		case ZEND_FAST_CALL:
+			if (op.extended_value == ZEND_FAST_CALL_FROM_FINALLY) {
+				fetch_type = "from_finally";
+			}
+			break;
+	}
+#endif
+
 	if (flags & OP_FETCH) {
-#ifdef ZEND_ENGINE_2
+#if defined(ZEND_ENGINE_2) || defined(ZEND_ENGINE_3)
 		switch (op.VLD_EXTENDED_VALUE(op2)) {
 			case ZEND_FETCH_GLOBAL:
 				fetch_type = "global";
@@ -578,12 +718,14 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 			case ZEND_FETCH_LOCAL:
 				fetch_type = "local";
 				break;
+#if PHP_VERSION_ID < 70100
 			case ZEND_FETCH_STATIC:
 				fetch_type = "static";
 				break;
 			case ZEND_FETCH_STATIC_MEMBER:
 				fetch_type = "static member";
 				break;
+#endif
 #ifdef ZEND_FETCH_GLOBAL_LOCK
 			case ZEND_FETCH_GLOBAL_LOCK:
 				fetch_type = "global lock";
@@ -598,7 +740,7 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 				fetch_type = "unknown";
 				break;
 		}
-#else 
+#else /* ZEND_ENGINE_1 */
 		if (op.op2.u.fetch_type == ZEND_FETCH_GLOBAL) {
 			fetch_type = "global";
 		} else if (op.op2.u.fetch_type == ZEND_FETCH_STATIC) {
@@ -634,9 +776,13 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 		vld_printf(stderr, "     ");
 	}
 
+#if PHP_VERSION_ID >= 70100
+	if ((flags & RES_USED) && op.result_type != IS_UNUSED) {
+#else
 	if ((flags & RES_USED) && !(op.VLD_EXTENDED_VALUE(result) & EXT_TYPE_UNUSED)) {
+#endif
 		VLD_PRINT(3, " RES[ ");
-		len = vld_dump_znode (NULL, res_type, op.result, base_address TSRMLS_CC);
+		len = vld_dump_znode (NULL, res_type, op.result, base_address, opa, nr TSRMLS_CC);
 		VLD_PRINT(3, " ]");
 		if (VLD_G(format)) {
 			if (len==0) {
@@ -651,9 +797,9 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 
 	if (flags & OP1_USED) {
 		VLD_PRINT(3, " OP1[ ");
-		vld_dump_znode (&print_sep, op1_type, op.op1, base_address TSRMLS_CC);
+		vld_dump_znode (&print_sep, op1_type, op.op1, base_address, opa, nr TSRMLS_CC);
 		VLD_PRINT(3, " ]");
-	}		
+	}
 	if (flags & OP2_USED) {
 		VLD_PRINT(3, " OP2[ ");
 		if (flags & OP2_INCLUDE) {
@@ -685,10 +831,11 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 					break;
 			}
 		} else {
-			vld_dump_znode (&print_sep, op2_type, op.op2, base_address TSRMLS_CC);
+			vld_dump_znode (&print_sep, op2_type, op.op2, base_address, opa, nr TSRMLS_CC);
 		}
 		VLD_PRINT(3, " ]");
 	}
+#if PHP_VERSION_ID < 70000
 	if (flags & OP2_BRK_CONT) {
 		long jmp;
 		zend_brk_cont_element *el;
@@ -703,9 +850,24 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 		vld_printf (stderr, ", ->%d", jmp);
 		VLD_PRINT(3, " ]");
 	}
+#endif
+	if (flags & EXT_VAL_JMP_ABS) {
+		VLD_PRINT(3, " EXT_JMP_ABS[ ");
+		vld_printf (stderr, ", ->%d", op.extended_value);
+		VLD_PRINT(3, " ]");
+	}
+	if (flags & EXT_VAL_JMP_REL) {
+		VLD_PRINT(3, " EXT_JMP_REL[ ");
+#if PHP_VERSION_ID >= 70000
+		vld_printf (stderr, ", ->%d", nr + ((int) op.extended_value / sizeof(zend_op)));
+#else
+		vld_printf (stderr, ", ->%d", nr + op.extended_value);
+#endif
+		VLD_PRINT(3, " ]");
+	}
 	if (flags & NOP2_OPNUM) {
 		zend_op next_op = op_ptr[nr+1];
-		vld_dump_znode (&print_sep, VLD_IS_OPNUM, next_op.op2, base_address TSRMLS_CC);
+		vld_dump_znode (&print_sep, VLD_IS_OPNUM, next_op.op2, base_address, opa, nr TSRMLS_CC);
 	}
 	vld_printf (stderr, "\n");
 }
@@ -716,6 +878,7 @@ void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set,
 void vld_dump_oparray(zend_op_array *opa TSRMLS_DC)
 {
 	unsigned int i;
+	int          j;
 	vld_set *set;
 	vld_branch_info *branch_info;
 	unsigned int base_address = (unsigned int)(zend_intptr_t)&(opa->opcodes[0]);
@@ -737,8 +900,8 @@ void vld_dump_oparray(zend_op_array *opa TSRMLS_DC)
 	}
 #ifdef IS_CV /* PHP >= 5.1 */
 	vld_printf (stderr, "compiled vars:  ");
-	for (i = 0; i < opa->last_var; i++) {
-		vld_printf (stderr, "!%d = $%s%s", i, OPARRAY_VAR_NAME(opa->vars[i]), ((i + 1) == opa->last_var) ? "\n" : ", ");
+	for (j = 0; j < opa->last_var; j++) {
+		vld_printf (stderr, "!%d = $%s%s", j, OPARRAY_VAR_NAME(opa->vars[j]), ((j + 1) == opa->last_var) ? "\n" : ", ");
 	}
 	if (!opa->last_var) {
 		vld_printf(stderr, "none\n");
@@ -770,6 +933,7 @@ void opt_set_nop (zend_op_array *opa, int nr)
 	opa->opcodes[nr].opcode = ZEND_NOP;
 }
 
+#if PHP_VERSION_ID < 70000
 zend_brk_cont_element* vld_find_brk_cont(int nest_levels, int array_offset, zend_op_array *op_array)
 {
 	zend_brk_cont_element *jmp_to;
@@ -780,14 +944,19 @@ zend_brk_cont_element* vld_find_brk_cont(int nest_levels, int array_offset, zend
 	} while (--nest_levels > 0);
 	return jmp_to;
 }
+#endif
 
 int vld_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *jmp2)
 {
+#if PHP_VERSION_ID < 70000 || ZEND_USE_ABS_JMP_ADDR
 	zend_op *base_address = &(opa->opcodes[0]);
+#endif
 
 	zend_op opcode = opa->opcodes[position];
 	if (opcode.opcode == ZEND_JMP) {
-#ifdef ZEND_ENGINE_2
+#if PHP_VERSION_ID >= 70000
+		*jmp1 = VLD_ZNODE_JMP_LINE(opcode.op1, position, base_address);
+#elif defined(ZEND_ENGINE_2)
 		*jmp1 = ((long) VLD_ZNODE_ELEM(opcode.op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
 #else
 		*jmp1 = opcode.op1.u.opline_num;
@@ -800,7 +969,9 @@ int vld_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *j
 		opcode.opcode == ZEND_JMPNZ_EX
 	) {
 		*jmp1 = position + 1;
-#ifdef ZEND_ENGINE_2
+#if PHP_VERSION_ID >= 70000
+		*jmp2 = VLD_ZNODE_JMP_LINE(opcode.op2, position, base_address);
+#elif defined(ZEND_ENGINE_2)
 		*jmp2 = ((long) VLD_ZNODE_ELEM(opcode.op2, jmp_addr) - (long) base_address) / sizeof(zend_op);
 #else
 		*jmp2 = opcode.op1.u.opline_num;
@@ -808,17 +979,19 @@ int vld_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *j
 		return 1;
 	} else if (opcode.opcode == ZEND_JMPZNZ) {
 #if PHP_VERSION_ID >= 70000
-		*jmp1 = VLD_ZNODE_ELEM(opcode.op2, jmp_addr) - base_address;
+		*jmp1 = VLD_ZNODE_JMP_LINE(opcode.op2, position, base_address);
+		*jmp2 = (int32_t) (position + ((int32_t)opcode.extended_value / sizeof(zend_op)));
 #else
 		*jmp1 = VLD_ZNODE_ELEM(opcode.op2, opline_num);
-#endif
 		*jmp2 = opcode.extended_value;
+#endif
 		return 1;
+#if PHP_VERSION_ID < 70000
 	} else if (opcode.opcode == ZEND_BRK || opcode.opcode == ZEND_CONT) {
 		zend_brk_cont_element *el;
 
 		if (VLD_TYPE(opcode.op2) == IS_CONST
-#if PHP_MAJOR_VERSION >= 5
+#ifdef ZEND_ENGINE_2
 		    && VLD_ZNODE_ELEM(opcode.op1, jmp_addr) != (zend_op*) 0xFFFFFFFF
 #endif
 		) {
@@ -830,10 +1003,19 @@ int vld_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *j
 			*jmp1 = opcode.opcode == ZEND_BRK ? el->brk : el->cont;
 			return 1;
 		}
+#endif
+#if PHP_VERSION_ID >= 70000
+	} else if (opcode.opcode == ZEND_FE_FETCH_R || opcode.opcode == ZEND_FE_FETCH_RW) {
+		*jmp1 = position + 1;
+		*jmp2 = position + (opcode.extended_value / sizeof(zend_op));
+		return 1;
+	} else if (opcode.opcode == ZEND_FE_RESET_R || opcode.opcode == ZEND_FE_RESET_RW) {
+#else
 	} else if (opcode.opcode == ZEND_FE_RESET || opcode.opcode == ZEND_FE_FETCH) {
+#endif
 		*jmp1 = position + 1;
 #if PHP_VERSION_ID >= 70000
-		*jmp2 = VLD_ZNODE_ELEM(opcode.op2, jmp_addr) - base_address;
+		*jmp2 = VLD_ZNODE_JMP_LINE(opcode.op2, position, base_address);
 #else
 		*jmp2 = VLD_ZNODE_ELEM(opcode.op2, opline_num);
 #endif
@@ -841,24 +1023,54 @@ int vld_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *j
 	} else if (opcode.opcode == ZEND_CATCH) {
 		*jmp1 = position + 1;
 #if PHP_VERSION_ID >= 50400
-        if (!opcode.result.num) {
+		if (!opcode.result.num) {
 #else
-        if (!opcode.op1.u.EA.type) {
+		if (!opcode.op1.u.EA.type) {
 #endif
-            *jmp2 = opcode.extended_value;
-            if (*jmp2 == *jmp1) {
-                *jmp2 = VLD_JMP_NOT_SET;
-            }
-        } else {
+#if PHP_VERSION_ID >= 70100
+			*jmp2 = position + (opcode.extended_value / sizeof(zend_op));
+#else
+			*jmp2 = opcode.extended_value;
+#endif
+			if (*jmp2 == *jmp1) {
+				*jmp2 = VLD_JMP_NOT_SET;
+			}
+		} else {
 			*jmp2 = VLD_JMP_EXIT;
 		}
 		return 1;
-#if (PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3)
+#if PHP_VERSION_ID >= 50300
 	} else if (opcode.opcode == ZEND_GOTO) {
+#if PHP_VERSION_ID >= 70000
+		*jmp1 = VLD_ZNODE_JMP_LINE(opcode.op1, position, base_address);
+#else
 		*jmp1 = ((long) VLD_ZNODE_ELEM(opcode.op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
+#endif
 		return 1;
 #endif
-	} else if (opcode.opcode == ZEND_EXIT || opcode.opcode == ZEND_THROW || opcode.opcode == ZEND_RETURN) {
+
+#if PHP_VERSION_ID >= 50500
+	} else if (opcode.opcode == ZEND_FAST_CALL) {
+#if PHP_VERSION_ID >= 70000
+		*jmp1 = VLD_ZNODE_JMP_LINE(opcode.op1, position, base_address);
+#else
+		*jmp1 = ((long) VLD_ZNODE_ELEM(opcode.op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
+#endif
+		*jmp2 = position + 1;
+		return 1;
+	} else if (opcode.opcode == ZEND_FAST_RET) {
+		*jmp1 = VLD_JMP_EXIT;
+		return 1;
+#endif
+
+	} else if (
+#if PHP_VERSION_ID >= 50500
+		opcode.opcode == ZEND_GENERATOR_RETURN ||
+#endif
+		opcode.opcode == ZEND_EXIT ||
+		opcode.opcode == ZEND_THROW ||
+		opcode.opcode == ZEND_RETURN
+	) {
 		*jmp1 = VLD_JMP_EXIT;
 		return 1;
 	}
@@ -919,7 +1131,11 @@ void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set,
 
 		/* See if we have a jump instruction */
 		if (vld_find_jump(opa, position, &jump_pos1, &jump_pos2)) {
-			VLD_PRINT1(1, "Jump found. Position 1 = %d", jump_pos1);
+			VLD_PRINT2(
+				1, "Jump found. (Code = %d) Position 1 = %d",
+				opa->opcodes[position].opcode,
+				jump_pos1
+			);
 			if (jump_pos2 != VLD_JMP_NOT_SET) {
 				VLD_PRINT1(1, ", Position 2 = %d\n", jump_pos2);
 			} else {
@@ -939,7 +1155,7 @@ void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set,
 			}
 			break;
 		}
-#ifdef ZEND_ENGINE_2
+#if defined(ZEND_ENGINE_2) || defined(ZEND_ENGINE_3)
 		/* See if we have a throw instruction */
 		if (opa->opcodes[position].opcode == ZEND_THROW) {
 			VLD_PRINT1(1, "Throw found at %d\n", position);
