@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 Derick Rethans                               |
+   | Copyright (c) 1997-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -66,6 +66,8 @@
 #define VLD_IS_OPNUM  1<<14
 #define VLD_IS_CLASS  1<<15
 #define OP2_INCLUDE   1<<16
+#define EXT_VAL_JMP_REL   1<<17
+#define EXT_VAL_JMP_ABS   1<<18
 
 typedef struct _op_usage {
 	char *name;
