@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 Derick Rethans                               |
+   | Copyright (c) 1997-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to the 2-Clause BSD license which is     |
    | available through the LICENSE file, or online at                     |
@@ -45,9 +45,24 @@ static void (*old_execute)(zend_op_array *op_array TSRMLS_DC);
 static void vld_execute(zend_op_array *op_array TSRMLS_DC);
 #endif
 
+/* {{{ forward declarations */
+static int vld_check_fe (zend_op_array *fe, zend_bool *have_fe TSRMLS_DC);
+static int vld_dump_fe (zend_op_array *fe APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key);
+#if defined(ZEND_ENGINE_3)
+static int vld_dump_cle (zend_class_entry *class_entry TSRMLS_DC);
+#elif defined(ZEND_ENGINE_2)
+static int vld_dump_cle (zend_class_entry **class_entry TSRMLS_DC);
+#else
+static int vld_dump_cle (zend_class_entry *class_entry TSRMLS_DC);
+#endif
+/* }}} */
+
+#if PHP_VERSION_ID < 50300
+# define ZEND_FE_END { NULL, NULL, NULL }
+#endif
 
 zend_function_entry vld_functions[] = {
-	{NULL, NULL, NULL}
+	ZEND_FE_END
 };
 
 
@@ -63,7 +78,7 @@ zend_module_entry vld_module_entry = {
 	PHP_RSHUTDOWN(vld),
 	PHP_MINFO(vld),
 #if ZEND_MODULE_API_NO >= 20010901
-	"0.13.0",
+	"0.14.0-dev",
 #endif
 	STANDARD_MODULE_PROPERTIES
 };
@@ -88,18 +103,18 @@ PHP_INI_BEGIN()
 	STD_PHP_INI_ENTRY("vld.dump_paths",   "1", PHP_INI_SYSTEM, OnUpdateBool, dump_paths,   zend_vld_globals, vld_globals)
 PHP_INI_END()
  
-static void vld_init_globals(zend_vld_globals *vld_globals)
+static void vld_init_globals(zend_vld_globals *vg)
 {
-	vld_globals->active       = 0;
-	vld_globals->skip_prepend = 0;
-	vld_globals->skip_append  = 0;
-	vld_globals->execute      = 1;
-	vld_globals->format       = 0;
-	vld_globals->col_sep	  = "\t";
-	vld_globals->path_dump_file = NULL;
-	vld_globals->dump_paths   = 1;
-	vld_globals->save_paths   = 0;
-	vld_globals->verbosity    = 1;
+	vg->active       = 0;
+	vg->skip_prepend = 0;
+	vg->skip_append  = 0;
+	vg->execute      = 1;
+	vg->format       = 0;
+	vg->col_sep	  = "\t";
+	vg->path_dump_file = NULL;
+	vg->dump_paths   = 1;
+	vg->save_paths   = 0;
+	vg->verbosity    = 1;
 }
 
 
@@ -203,12 +218,39 @@ PHP_MINFO_FUNCTION(vld)
 
 }
 
+/* {{{ PHP 7 wrappers */
+#if defined(ZEND_ENGINE_3)
+
+#define VLD_WRAP_PHP7(name) name ## _wrapper
+
+static int vld_check_fe_wrapper (zval *el, zend_bool *have_fe TSRMLS_DC)
+{
+	return vld_check_fe((zend_op_array *) Z_PTR_P(el), have_fe TSRMLS_CC);
+}
+
+static int vld_dump_fe_wrapper(zval *el APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
+{
+	return vld_dump_fe((zend_op_array *) Z_PTR_P(el) APPLY_TSRMLS_CC, num_args, args, hash_key);
+}
+
+static int vld_dump_cle_wrapper (zval *el TSRMLS_DC)
+{
+	return vld_dump_cle((zend_class_entry *) Z_PTR_P(el) TSRMLS_CC);
+}
+#else
+
+#define VLD_WRAP_PHP7(name) name
+
+#endif
+/* }}} */
+
 int vld_printf(FILE *stream, const char* fmt, ...)
 {
 	char *message;
 	int len;
 	va_list args;
-	int i = 0, j = 0;
+	int i = 0;
+	size_t j = 0;
 	char *ptr;
 	const char EOL='\n';
 	TSRMLS_FETCH();
@@ -252,9 +294,11 @@ static int vld_dump_fe (zend_op_array *fe APPLY_TSRMLS_DC, int num_args, va_list
 #endif
 	if (fe->type == ZEND_USER_FUNCTION) {
 		ZVAL_VALUE_STRING_TYPE *new_str;
+#if PHP_VERSION_ID < 70000
 		int new_len;
+#endif
 
-		new_str = php_url_encode(ZHASHKEYSTR(hash_key), ZHASHKEYLEN(hash_key) - 1 PHP_URLENCODE_NEW_LEN(new_len));
+		new_str = php_url_encode(ZHASHKEYSTR(hash_key), ZHASHKEYLEN(hash_key) PHP_URLENCODE_NEW_LEN(new_len));
 		vld_printf(stderr, "Function %s:\n", ZSTRING_VALUE(new_str));
 		vld_dump_oparray(fe TSRMLS_CC);
 		vld_printf(stderr, "End of function %s\n\n", ZSTRING_VALUE(new_str));
@@ -264,7 +308,8 @@ static int vld_dump_fe (zend_op_array *fe APPLY_TSRMLS_DC, int num_args, va_list
 	return ZEND_HASH_APPLY_KEEP;
 }
 
-#ifdef ZEND_ENGINE_2
+
+#if defined(ZEND_ENGINE_2)
 static int vld_dump_cle (zend_class_entry **class_entry TSRMLS_DC)
 #else
 static int vld_dump_cle (zend_class_entry *class_entry TSRMLS_DC)
@@ -273,7 +318,7 @@ static int vld_dump_cle (zend_class_entry *class_entry TSRMLS_DC)
 	zend_class_entry *ce;
 	zend_bool have_fe = 0;
 
-#ifdef ZEND_ENGINE_2
+#if defined(ZEND_ENGINE_2)
 	ce = *class_entry;
 #else
 	ce = class_entry;
@@ -284,10 +329,11 @@ static int vld_dump_cle (zend_class_entry *class_entry TSRMLS_DC)
 			fprintf(VLD_G(path_dump_file), "subgraph cluster_class_%s { label=\"class %s\";\n", ZSTRING_VALUE(ce->name), ZSTRING_VALUE(ce->name));
 		}
 
-		zend_hash_apply_with_argument(&ce->function_table, (apply_func_arg_t) vld_check_fe, (void *)&have_fe TSRMLS_CC);
+		zend_hash_apply_with_argument(&ce->function_table, (apply_func_arg_t) VLD_WRAP_PHP7(vld_check_fe), (void *)&have_fe TSRMLS_CC);
+
 		if (have_fe) {
 			vld_printf(stderr, "Class %s:\n", ZSTRING_VALUE(ce->name));
-			zend_hash_apply_with_arguments(&ce->function_table APPLY_TSRMLS_CC, (apply_func_args_t) vld_dump_fe, 0);
+			zend_hash_apply_with_arguments(&ce->function_table APPLY_TSRMLS_CC, (apply_func_args_t) VLD_WRAP_PHP7(vld_dump_fe), 0);
 			vld_printf(stderr, "End of class %s.\n\n", ZSTRING_VALUE(ce->name));
 		} else {
 			vld_printf(stderr, "Class %s: [no user functions]\n", ZSTRING_VALUE(ce->name));
@@ -301,6 +347,7 @@ static int vld_dump_cle (zend_class_entry *class_entry TSRMLS_DC)
 	return ZEND_HASH_APPLY_KEEP;
 }
 
+
 /* {{{ zend_op_array vld_compile_file (file_handle, type)
  *    This function provides a hook for compilation */
 static zend_op_array *vld_compile_file(zend_file_handle *file_handle, int type TSRMLS_DC)
@@ -327,14 +374,14 @@ static zend_op_array *vld_compile_file(zend_file_handle *file_handle, int type T
 	op_array = old_compile_file (file_handle, type TSRMLS_CC);
 
 	if (VLD_G(path_dump_file)) {
-		fprintf(VLD_G(path_dump_file), "subgraph cluster_file_%08x { label=\"file %s\";\n", op_array, op_array->filename ? ZSTRING_VALUE(op_array->filename) : "__main");
+		fprintf(VLD_G(path_dump_file), "subgraph cluster_file_%p { label=\"file %s\";\n", op_array, op_array->filename ? ZSTRING_VALUE(op_array->filename) : "__main");
 	}
 	if (op_array) {
 		vld_dump_oparray (op_array TSRMLS_CC);
 	}
 
-	zend_hash_apply_with_arguments (CG(function_table) APPLY_TSRMLS_CC, (apply_func_args_t) vld_dump_fe, 0);
-	zend_hash_apply (CG(class_table), (apply_func_t) vld_dump_cle TSRMLS_CC);
+	zend_hash_apply_with_arguments (CG(function_table) APPLY_TSRMLS_CC, (apply_func_args_t) VLD_WRAP_PHP7(vld_dump_fe), 0);
+	zend_hash_apply (CG(class_table), (apply_func_t) VLD_WRAP_PHP7(vld_dump_cle) TSRMLS_CC);
 
 	if (VLD_G(path_dump_file)) {
 		fprintf(VLD_G(path_dump_file), "}\n");
@@ -355,8 +402,13 @@ static zend_op_array *vld_compile_string(zval *source_string, char *filename TSR
 	if (op_array) {
 		vld_dump_oparray (op_array TSRMLS_CC);
 
+#if defined(ZEND_ENGINE_3)
+		zend_hash_apply_with_arguments (CG(function_table) APPLY_TSRMLS_CC, (apply_func_args_t) vld_dump_fe_wrapper, 0);
+		zend_hash_apply (CG(class_table), (apply_func_t) vld_dump_cle_wrapper TSRMLS_CC);
+#else
 		zend_hash_apply_with_arguments (CG(function_table) APPLY_TSRMLS_CC, (apply_func_args_t) vld_dump_fe, 0);
 		zend_hash_apply (CG(class_table), (apply_func_t) vld_dump_cle TSRMLS_CC);
+#endif
 	}
 
 	return op_array;
